*=====================================================================
*     MDynaMix v.4.3 
*     Part 2
*
*     Input data
C
C     This file contains subroutines responsible for input of data:
C
C     1. INPUT  - input of simulation parameters
C     2. READMOL - reads molecular structure and force field parameters
C     3. TAKESTR (char*128) - read line as character line
C     4. HYDRO - read hydrogen bonds parameters
C        
C     5. Other units are supplementary functions:
C
C        ANGLE   - calculate angles
C        PARSES  - perform character input in free format  
*
*    1. Input of simulations parameters
*    ----------------------------------
C================ INPUT ===============================================
	subroutine INPUT   
	include "prcm.h"
C   maximal length of input lines (the same numbers in the next 2 lines) 
	parameter (LSTR=128)   
	character*128 STR,TAKESTR,AUX,FPOT  
	integer ISPT(MAXPOT),JSPT(MAXPOT)
	IE=0                 ! error index
C  In the case of parallel execution this line is changed to
C  provide input from file "md.input" (see file input.patch)
	IO=5                 ! input channel (fortran standard input) 
C   The following two lines give a sample of how data input is organized:
C   First function TAKESTR gets next line which does not have # as
C   a first symbol; then operator "read" reads data from character line, 
C   normaly using free format
*   1.1 Output details level
	STR=TAKESTR(IO,IE) 
	read(STR,*,err=99,end=99)IPRINT                
	if(IPRINT.ge.7)write(*,*)' Checking input: '
*   1.2 Base file name
	STR=TAKESTR(IO,IE)
	do I=1,128
	  if(STR(I:I).ne.' ')go to 1
	end do
 1	LD=LENG(STR,128)
	FNAME=STR(I:LD)    
	LDF=LD-I+1
	if(IPRINT.ge.7)write(*,*)' Basis file name ',FNAME
*   1.3  Path to molecular data base
	STR=TAKESTR(IO,IE)
	do I=1,128
	  if(STR(I:I).ne.' ')go to 2
	end do
 2	LD=LENG(STR,128)
	PATHDB=STR(I:LD)    
	if(IPRINT.ge.7)write(*,*)' Molecular database path ',PATHDB(1:LD)
*   1.4  Restart controll
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LRST,LDMP,LINPUT,L0AVS   
	if(IPRINT.ge.7.and.LRST)
     +write(*,*)' restart from ',FNAME(1:LDF)//'.dmp'
	if(IPRINT.ge.7.and.LDMP)write(*,*)' restart file will be dumped'
	if(IPRINT.ge.7.and.LINPUT)write(*,*)
     +' only check input or results'
	if(IPRINT.ge.7.and.L0AVS)write(*,*)' count cpu time from 0'
*   1.5  Defene statistical ensemble type
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LNVT,LNPT,LSEP  
	if(.not.LNPT)LSEP=.false.
	if(IPRINT.ge.7.and.LNVT)write(*,*)' constant temperature'
	if(IPRINT.ge.7.and.LNPT)write(*,*)' constant pressure'
	if(IPRINT.ge.7.and.LSEP)write(*,*)
     +' separate scaling box of sizes'
*    1.6 Number of molecular specii
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)NTYPES
	if(IPRINT.ge.7)write(*,*)' number of molecule species: ',NTYPES
      IF(NTYPES.GT.NTPS) STOP '!!! INCREASE NTPS !'
*    1.7  Molecules names
	STR=TAKESTR(IO,IE)
        call PARSES(STR,NAMOL,NTYPES,LSTR,ie)
	if(IPRINT.ge.7)write(*,*)' names of molecules of each specii: ',
     + (NAMOL(I),I=1,NTYPES)
*    1.8  Number of molecules
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(NSPEC(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)' number of molecules of each specii: ',
     + (NSPEC(I),I=1,NTYPES)
*    1.9 Intramolecular interactions parameters
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(L15NB(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)' nonbonded intramolec.interactions : ',
     + (L15NB(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(L14NB(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)' 1-4 intramolec.interactions : ',
     + (L14NB(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(C14LJ(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)
     +' Scaling factors for 1-4 LJ intramolec.interactions : ',
     + (C14LJ(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(C14EL(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)
     +' Scaling factors for 1-4 electrost. intramolec.interactions:',
     + (C14EL(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(IPOT(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)' intramolecular potential type: ',
     + (IPOT(I),I=1,NTYPES)
*   1.10 Thermodynamical data
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)TRTEMP,RHO,TRPRES
	if(IPRINT.ge.7)write(*,*)' temperature ',TRTEMP
	if(IPRINT.ge.7)write(*,*)' pressure ',TRPRES
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)BOXL,BOYL,BOZL,ICELL
	LOCT=.false.
	LHEX=.false.
	if(ICELL.eq.1)LOCT=.true.
	if(ICELL.eq.2)LHEX=.true.
C  Actual choice of box size or density will be later in UNITS (setup.f)
	if(LHEX)BOYL=cos30*BOXL
	if(IPRINT.ge.7)then
	  if(BOXL*BOYL*BOZL.le.0)then
	    write(*,*)' density ',RHO
	  else
	    write(*,*)' box sizes: ',BOXL,BOYL,BOZL
	  end if
	end if
*   1.11   Time step
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)DT,NFREQ
	if(IPRINT.ge.7)write(*,*)' Large time step ',DT
	if(IPRINT.ge.7)write(*,*)' Small time step ',DT/(NFREQ+1.d-14)
	STR=TAKESTR(IO,IE)
*   1.12 Number of MD steps
	read(STR,*,err=99,end=99)NSTEPS,NAVER,IAVER,NDUMP
	if(IPRINT.ge.7)write(*,*)' Total number of MD time steps ',NSTEPS
	if(IPRINT.ge.7)write(*,*)
     +' MD time steps for intermediate averaging',NAVER
	if(IPRINT.ge.7)write(*,*)' Take averages each ',IAVER,' steps'
	if(IPRINT.ge.7)write(*,*)' Restart file after ',NDUMP,' steps'
*   1.13 Thermostat parameters
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)QT,QPR,LSCLT,TDELT
	if(IPRINT.ge.7)write(*,*)' Termostat parameters ',QT,QPR
	if(IPRINT.ge.7.and.LSCLT)write(*,*)
     +' Forsible keeping temperature with allowed fluctuations ',TDELT
	if(LSCLT.and.LNVT)then
	  LSCLT=.false.
	  write(*,*)'!!! Forsible keeping temperature switched off ',
     +'because Nose temperature control was specified'
	end if
	STR=TAKESTR(IO,IE)
*   1.14   Cutoff controll
	read(STR,*,err=99,end=99)RCUT,SHORT,ICHNB
	if(IPRINT.ge.7)write(*,*)' Cut off radius ',RCUT
	if(IPRINT.ge.7)write(*,*)' Cut off for fast forces ',SHORT
	if(IPRINT.ge.7)write(*,*)' Check neighbours after ',ICHNB,
     +' steps'
*   1.15 Long-range electrostatic controll
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)ALPHA,FEXP
	if(IPRINT.ge.7)write(*,*)' Ewald parameters ',ALPHA,FEXP
*   1.16  which molecules move
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(LMOVE(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)'move molecules: ',(LMOVE(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LINIT
	if(IPRINT.ge.7.and.LINIT)write(*,*)
     +'recalculate intramolec. neigbours list '
*   1.17  Constrained dynamics ?
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LSHEJK
	if(LSHEJK)
     +read(STR,*,err=99,end=99)LSHEJK,TOLER,(ISHEJK(I),I=1,NTYPES)
	if(IPRINT.ge.7.and.LSHEJK)then
          write(*,*)'Constrain dynamics. TolER= ',TOLER
	  write(*,*)'This molecules will be constrained:'
	  do I=1,NTYPES
	     if(ISHEJK(I).ne.0)write(*,*)NAMOL(I)
	  end do
	end if
*   1.18 How to start up
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)ICHECK,L0VS
	if(IPRINT.ge.7)write(*,*)' Initial state parameter ',ICHECK
	if(IPRINT.ge.7.and.L0VS)write(*,*)' set velocities to 0 '
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(IINIT(I),I=1,NTYPES)
	if(IPRINT.ge.7)write(*,*)' Initial orientation parameter',
     +(IINIT(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)RDNA,LRR
	if(LRR)then
	   read(STR,*,err=99,end=99)RDNA,LRR,RLR,filref
	else
	   RLR=1.
	end if
	if(IPRINT.ge.7.and.(ICHECK.eq.2.or.ICHECK.eq.3))write(*,*)
     +' Radius of the hole for a large molecule',RDNA 
	if(IPRINT.ge.7.and.LRR)then
	   write(*,*)' Standard structure taken from file ',filref
	   write(*,*)' Deviation from "standard" structure',RLR
	end if
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LCHT,LCHP
	if(IPRINT.ge.7.and.LCHT)write(*,*)
     +' Set temperature from the input file'
	if(IPRINT.ge.7.and.LCHT)write(*,*)
     +' Set density or box size from the input file'
*  1.19  Output controll
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)IHIST,LXMOL
	if(IPRINT.ge.7)write(*,*)' final averaging from ',IHIST,
     +' fragment '
	if(IPRINT.ge.7.and.LXMOL)write(*,*)
     +' Dump configuration for XMOL program'
*   1.20
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LSCFT,FTSCL
	if(IPRINT.ge.7.and.LSCFT)write(*,*)' Scale forces to ',
     +  FTSCL,'i.u.'
*   1.21  RDF controll
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LGR,LGDMP,LGRST
	if(IPRINT.ge.7.and.LGR)write(*,*)' calculate RDF'
	if(IPRINT.ge.7.and.LGRST)write(*,*)' restart RDF file'
	if(IPRINT.ge.7.and.LGDMP)write(*,*)' dump RDF file'
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)ALLSTS,RDFCUT,MAX
	if(IPRINT.ge.7.and.ALLSTS)write(*,*)' RDF for all site pairs'
	if(IPRINT.ge.7)write(*,*)' RDF cut off ',RDFCUT 
	if(IPRINT.ge.7)write(*,*)' RDF resolution ',RDFCUT/MAX 
*   1.22  Trajectory dump controll
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)ITREK,TTREK,LTREK
	if(IPRINT.ge.7.and.ITREK.gt.0)write(*,*)
     +' Dump configurations each ',TTREK,' s ',LTREK,' conf. per file'
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(LIST(I),I=1,NTYPES)
	if(IPRINT.ge.7.and.ITREK.gt.0)write(*,*)
     +' Include molecules: ',(LIST(I),I=1,NTYPES)
*   1.23  Time correlation functions
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)LCF,LCFRST,LCFDMP,NSTEG,JUMP
	if(IPRINT.ge.7.and.LCF)write(*,*)' calculate TCF'
	if(IPRINT.ge.7.and.LCFRST)write(*,*)' restart TCF file'
	if(IPRINT.ge.7.and.LCFDMP)write(*,*)' dump TCF file'
	if(IPRINT.ge.7.and.LCF)write(*,*)' TCF for ',NSTEG,
     + ' steps with jump = ',JUMP
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(ITCF(I),I=1,NTCF)
	if(IPRINT.ge.7.and.LCF)write(*,*)' which tcf calculate:',
     +(ITCF(I),I=1,NTCF)
	STR=TAKESTR(IO,IE)
	read(STR,*,err=99,end=99)(N1(I),I=1,NTYPES)
	STR=TAKESTR(IO,IE)                              
	read(STR,*,err=99,end=99)(N2(I),I=1,NTYPES)
	do I=1,NTYPES
	  if(N1(I).le.0)N1(I)=1
	  if(N2(I).le.0)N2(I)=1
	end do
	if(IPRINT.ge.7.and.LCF)write(*,*)
     +' unit vector for reorientation tcf:'
	if(IPRINT.ge.7.and.LCF)write(*,*) (N1(I),I=1,NTYPES)
	if(IPRINT.ge.7.and.LCF)write(*,*) (N2(I),I=1,NTYPES)
*   1.24  Optional parameters (for future)
*   1.24.1 Default values
C  ISTAV=1/0  - calculate or not average bond length.angle,torsions 
        ISTAV=1
C  Dump bond list 
	LBONDL=.false.
C  Dump COM coordinates
	LCOMD=.false.
	LGATHER=.false.
	LSCTD=.false.
	STR=TAKESTR(IO,IE)
	if(STR(1:3).eq.'add')then
	   AUX=STR(5:LSTR)
	   read(AUX,*,err=99,END=99)IADD
	   if(TASKID.eq.MAST)
     +       write(*,*)' reading ',IADD,' extra parameters...'
	   do I=1,IADD
	     STR=TAKESTR(IO,IE)
*   1.24.2  Signal formatted form of restart file
C           (this is to transfer a run to computer with another architecture) 
	     if(STR(1:6).eq.'inputf')then
	       LFORMI=.true.
	     else if(STR(1:7).eq.'outputf')then
	       LFORMO=.true.
*   1.24.3 
	     else if(STR(1:6).eq.'extpot')then
	        STR=TAKESTR(IO,IE)
C  NFP - Number of files with potentials
C  RP0,DRP,NPO - distances determined as R=RP0+(I-1)*DRP, I=1,...,NPO
		read(STR,*)NFP,RP0,DRP,NPO
		do J=1,NFP
		  STR=TAKESTR(IO,IE)
	          do IC=1,128
		    if(STR(IC:IC).ne.' ')go to 101
	          end do
 101		  LD=LENG(STR,128)
		  FPOT=STR(IC:LD)    
		  STR=TAKESTR(IO,IE)
C   NPOT - number of potentials in the file
		  read(STR,*)NPOT
		  do K=1,NPOT
		    STR=TAKESTR(IO,IE)
C   ICOL - num of colomn in the file;  ISPT,JSPT - site numbers
		    read(STR,*)ICOL,ISPT(K),JSPT(K)
		  end do
		end do
	     else if(STR(1:6).eq.'no ave')then
*   1.24.4
*   Do not calculate structural averages (bond lengths, angles, torsions)
		ISTAV=0
	     else if(STR(1:6).eq.'ext_el')then
*   1.24.5
*   External field
C   Amplitude EXAMPL in V/cm, EXFREQ in Hz.
		IEXT=1
		STR=TAKESTR(IO,IE)
		read(STR,*)EXAMPL,EXFREQ
		if(IPRINT.ge.7)write(*,*)
     +'External filed ',EXAMPL,' Frequency (hz) ',EXFREQ
*   1.24.6 
*   Generate bond list
             else if(STR(1:6).eq.'bond l')then
		LBONDL=.true.
*   1.24.7
*   Dump COM coordinates instead of XMOL format
             else if(STR(1:7).eq.'comdump')then
		LCOMD=.true.
*   1.24.8
*   Use Kong combination rules instead of Lorentz-Berthelot
             else if(STR(1:4).eq.'kong')then
		LKONG=.true.
*   1.24.9 
*   Gather molecule in the same cell
	     else if(STR(1:6).eq.'gather')then
		LGATHER=.true.
*   1.24.10
*   Separate temperature control of each molecular species
	     else if(STR(1:5).eq.'T-sep')then
		LSCTD=.true.
	     end if
	   end do
	   STR=TAKESTR(IO,IE)
	end if
*    1.25  Number of RDF-s 
	read(STR,*,err=99,end=99)MAXRDF
	if(TASKID.eq.MAST)write(*,*)' Read all input data '
 	return   
*   1.25  Diagnos               
 99	IE=99
	STR=TAKESTR(IO,IE)
	write(*,*)' Reboot in 30 sec '
	end
C=====================================================================
*
*    2. Read molecular structure and force field
*    -------------------------------------------
*
C    This subroutine read the complete information about the molecule 
C    of type NTYP from the .mol file located in molecular database. 
C    It containes initial configuration of the molecule, atom types, 
C    names, charges, LJ parameters, list of covalent bonds, angles, 
C    dihedral angles with corresponding force field parameters.
C    Only in a few cases these data may be overwritten by the data
C    from the input file (see subroutine INPUT)
C
C    Information how to write .mol files are given in README file
C
C================ READMOL =============================================
C
      subroutine READMOL(NTYP)
      include "prcm.h"
      DIMENSION CM(3)
      character namn*32,STR*128,FIL*128,TAKESTR*128,NAMH*4,NMF*5
      data NX/0/
*
*  2.1 Open the .mol file
*  ----------------------
      FIL=' '
C  molecule name                    
      NAMN=NAMOL(NTYP)
      IE=0
      NX0=NX
      LD=LENG(PATHDB,116)
C  path
      FIL(1:LD)=PATHDB(1:LD)
      FIL(LD+1:LD+1)='/'
      LN=LENG(NAMN,32)
C  add file name
      FIL(LD+2:LD+LN+1)=NAMN(1:LN)
C  add extension .mol
      FIL(LD+LN+2:LD+LN+5)='.mol'
      open(unit=12,file=fil,status='old',err=999)
      if(IPRINT.ge.5)
     +PRINT * ,'*** MOLECULE ',NAMN,',  Type  No.',NTYP
*
*  2.2 Read information about the atoms
*  ------------------------------------
        IE=0
	STR=TAKESTR(12,IE)
C  Number of atoms
	read(STR,*,err=99,end=99)NSTS
	if(IPRINT.ge.6)PRINT * ,'*** NR OF SITES:                ',NSTS
	NSITS(NTYP)=NSTS
	NSBEG=NX+1 
C  check unit vector from the input file             
	if(LCF)then 
	  if(N1(NTYP).gt.NSTS)then  
	    N1(NTYP)=1
	    write(*,*)
     +    '!!! Too large site number for unit vector. Set to 1'
	  end if  
	  if(N2(NTYP).gt.NSTS)then  
	    N2(NTYP)=NSTS
	    write(*,*)
     +    '!!! Too large site number for unit vector. Set to ',NSTS
	  end if  
      end if
C   Cycle over atoms
      do I=1,NSTS
	  STR=TAKESTR(12,IE)
	  IX=NX+I
	  NM(IX)=STR(1:4)
	  read(STR(5:80),*,err=99,end=99)(R(IX,J),J=1,3),
     +    MASS(IX),CHARGE(IX),SIGMA(IX),EPSIL(IX)
          if(IPRINT.ge.6)
     +      write(*,'(a5,a4,a4,3f7.3,a3,f8.4,a3,f6.3,a5,f8.4,a4,f9.4)')
     +      'atom ',NM(IX),'  R=',(R(IX,J),J=1,3),
     +      ' M=',MASS(IX),' Q=',CHARGE(IX),' si=',
     +      SIGMA(IX),' eps=',EPSIL(IX)
      end do
      NSEND=NX+NSTS
*
*  2.3 Make local coordinate system COM at zero
*  -------------------------------------------- 
      SUMM       = 0.D0
      DO IS      = NSBEG,NSEND
        SUMM       = SUMM+MASS(IS)
      END DO! OF IS
C  This is total mass of the molecule in atomic units
      SUMMAS(NTYP) = SUMM 
      CM(1)      = 0.D0
      CM(2)      = 0.D0
      CM(3)      = 0.D0
      DO IS      = NSBEG,NSEND
        CM(1)      = CM(1)+R(IS,1)*MASS(IS)
        CM(2)      = CM(2)+R(IS,2)*MASS(IS)
        CM(3)      = CM(3)+R(IS,3)*MASS(IS)
      END DO! OF IS
      CM(1)      = CM(1)/SUMM
      CM(2)      = CM(2)/SUMM
      CM(3)      = CM(3)/SUMM
C  Set local atom coordinate so that molecular center of mass was zero.
C  Exceptions: molecules with IINIT=1 (see p. 1.18 this file) which are
C  put in the cell as they are
      if(IINIT(NTYP).ne.1)then
        if(IPRINT.ge.6)write(*,*)' Setting local COM to 0.  Type  ',
     &    NAMOL(NTYP)
        DO K       = 1,3
          DO IS      = NSBEG,NSEND
            R(IS,K)    = R(IS,K)-CM(K)
          END DO! OF IS
        END DO! OF I
      end if
C  Print extra information
      if(IPRINT.ge.8)then
        write(*,*)'*** Molecular GEOMETRY (C.O.M. IN ORIGIN): '
        DO IS      = NSBEG,NSEND
          write(*,'(I4,3x,A4,3X,3(1X,F7.3))')
     +    IS,NM(IS),R(IS,1),R(IS,2),R(IS,3)
        END DO! OF IS
      end if
      if(IPRINT.ge.10)then
        write(*,*)
        write(*,*)'*** INTER ATOMIC DISTANCES '
        do IS=NSBEG,NSEND
	  do JS=IS+1,NSEND
	    RR2=0.
	    do K=1,3
	      RR2=RR2+(R(IS,K)-R(JS,K))**2
	    end do
 	    RR=sqrt(RR2)
            write(*,'(3a4,2x,f9.4)')NM(IS),' -> ',NM(JS),RR
	  end do
	end do
      end if
*
*  2.4 Output of the reference
*  ---------------------------
      STR=TAKESTR(12,IE)
      read(STR,*,err=99,end=99)NSR
      do I=1,NSR
	STR=TAKESTR(12,IE)
	if(IPRINT.ge.6)write(*,'(a80)')STR
      end do
*
*  2.5 Input bonds information
*  --------------------------- 
      STR=TAKESTR(12,IE)
C  Number of bonds
      read(STR,*,err=99,end=99)NBD
      NRB(NTYP)=NBD
      do K=NRBB+1,NRBB+NBD
	STR=TAKESTR(12,IE)
	read(STR,*,err=36,end=36)ID(K),IB(K),JB(K),RB(K),FB(K),
     + DB(K),RO(K)
	go to 37
 36	read(STR,*,err=36,end=36)ID(K),IB(K),JB(K),RB(K),FB(K)
	DB(K)=0.
	RO(K)=0.
	ID(K)=0
 37	continue
C  If bond length is zero in the input file, it is set from the
C  atom coordinates
	if(RB(K).le.0.d0)RB(K)=sqrt((R(IB(K),1)-R(JB(K),1))**2+
     +  (R(IB(K),2)-R(JB(K),2))**2+(R(IB(K),3)-R(JB(K),3))**2)
      end do
      if(IPRINT.ge.8.and.NBD.gt.0)then
        write(*,*) 
        PRINT "('*** PARAMETERS FOR ',I3,'  BONDS')",NBD
        PRINT *,'*** HARMONIC POTENTIALS:'
        IDTOT      = 0
        DO M       = NRBB+1,NRBB+NBD
          if(ID(M).ne.0)IDTOT=IDTOT+1
           write(*,'(a5,a4,a1,a4,2I6,a,f8.4,a,f8.2)')
     +    'Bond ',NM(IB(M)+NX),'-',NM(JB(M)+NX),IB(M),JB(M),' Req.=',
     +    RB(M),'   Force=',FB(M)
        END DO! OF M
        IF(IDTOT.GT.0) THEN
          write(*,*)
          write(*,*)'*** MORSE POTENTIALS:'
          DO M       = NRBB+1,NRBB+NRB(NTYP)
            if(ID(M).ne.0)
     +      PRINT "('*** BOND: ',I3,'  - ',I3,'  RO: ',F7.3,'  DISS  ',
     X      'ENERGY  :',F9.3)",IB(M),JB(M),RO(M),DB(M)
          END DO! OF M
        END IF
      end if
*
*  2.6 Input angles information
*  ----------------------------
      STR=TAKESTR(12,IE)
      read(STR,*,err=99,end=99)NAN
      NRA(NTYP)=NAN   
      do K=NRAA+1,NRAA+NAN
	STR=TAKESTR(12,IE)
	read(STR,*,err=99,end=99)IA(K),JA(K),KA(K),RA(K),FA(K)
	if(RA(K).le.0)RA(K)=ANGLG(R,IA(K),JA(K),KA(K),NS)*TODGR
      end do
      if(IPRINT.ge.8.and.NAN.gt.0)then
	PRINT *
        PRINT *,'*** COVALENT ANGLES '
	do K=NRAA+1,NRAA+NAN
          PRINT "('***',3i6,3(1X,A4),'  ->',F9.3,'  Force c.',f9.3)",
     +       IA(K) ,   JA(K) ,   KA(K) ,
     +    NM(IA(K)+NX),NM(JA(K)+NX),NM(KA(K)+NX),RA(K),FA(K)        
	end do
      end if
*
*  2.7 Input torsional angles information
*  -------------------------------------- 
C  (potential of standard 1+cos(al+n*f) type )
      STR=TAKESTR(12,IE)
      read(STR,*,err=99,end=99)NTT       
      NRT(NTYP)=NTT
      if(NTT.gt.0)then
	do K=NRTT+1,NRTT+NTT
	  ITORS(K)=0
	  STR=TAKESTR(12,IE)
C  Input AMBER torsion angle parameters
	  read(STR,*,err=99,end=99)
     +	  IT(K),JT(K),KT(K),LT(K),RT(K),FT(K),NMUL(K)
C  Set parameters to other types of torsions to zero.
	  FT2(K)=0.d0
	  FT3(K)=0.d0
	  FT4(K)=0.d0
	  FT5(K)=0.d0
	end do
	if(IPRINT.ge.8)then
	  PRINT *
          PRINT *,'*** DIHEDRALS '
	  write(*,*)'  1+cos(nf) type:'
	  do K=NRTT+1,NRTT+NTT
            PRINT"(4I6,4(1X,A4),' ->',I2,F8.2,'  F.c.',f9.3)"
     +      ,   IT(K),JT(K),KT(K),LT(K)
     +         ,NM(IT(K)+NX),NM(JT(K)+NX),NM(KT(K)+NX),NM(LT(K)+NX),
     +        NMUL(K),RT(K),FT(K)        
	  end do
	end if
      end if                 
      NRBB         = NRBB+NRB(NTYP)
      NRAA         = NRAA+NRA(NTYP)
      NRTT         = NRTT+NRT(NTYP)
*
*  2.8 Optional force field terms
*  ------------------------------
C  Optional force field terms may be specified after the list of torsions.
C  The corresponding fields begin from a key word, followed by the number
C  of the external parameters of this type and their list.
C  The number of types of external parameters may be increased in 
C  future releases   
 100  IE=-1
      STR=TAKESTR(12,IE)  
      if(IE.lt.0)go to 20
      do I=1,72
	if(STR(I:I).ne.' ')go to 105
      end do
      go to 100
 105  NMF=STR(I:I+4)
*  2.8.1  MM3 type torsion angle  (tors1)
C         Sum(Kn*cos(n*f)) , n=1,2,3
	if(NMF.eq.'Tors1'.or.NMF.eq.'tors1'.or.NMF.eq.'TORS1')then	
        STR=TAKESTR(12,IE)
	  read(STR,*,err=99,end=99)NTT       
	  NRT(NTYP)=NRT(NTYP)+NTT
	  if(NTT.gt.0)then
	    do K=NRTT+1,NRTT+NTT  
	      ITORS(K)=1
	      STR=TAKESTR(12,IE)
	      read(STR,*,err=99,end=99)
     +	    IT(K),JT(K),KT(K),LT(K),FT1(K),FT2(K),FT3(K)
	      NMUL(K)=0   
	      FT(K)=0.d0 
	      RT(K)=0.d0 
	      RT(K)=0.d0 
	      FT4(K)=0.d0
	      FT5(K)=0.d0
	    end do
	    if(IPRINT.ge.8)then
	      PRINT *
            PRINT *,'*** DIHEDRALS '
	      write(*,*)'  Sum(1+Cos(nf)) type'
	      do K=NRTT+1,NRTT+NTT
              PRINT"(4I6,4(1X,A4),' -> ',3f9.3)",
     +        IT(K),JT(K),KT(K),LT(K),
     +        NM(IT(K)+NX),NM(JT(K)+NX),NM(KT(K)+NX),NM(LT(K)+NX),
     +        FT1(K),FT2(K),FT3(K)        
	        FT1(K)=0.5*FT1(K)
	        FT2(K)=0.5*FT2(K)
	        FT3(K)=0.5*FT3(K)
	      end do
	    end if
          NRTT         = NRTT+NTT
	  end if
*  2.8.2   Dihedral of Sum(cos(f)**n) type   (tors5)
	else if(NMF.eq.'Tors5'.or.NMF.eq.'tors5'.or.NMF.eq.'TORS5')then	
        STR=TAKESTR(12,IE)
	  read(STR,*,err=99,end=99)NTT       
	  NRT(NTYP)=NRT(NTYP)+NTT
	  if(NTT.gt.0)then
	    do K=NRTT+1,NRTT+NTT  
	      ITORS(K)=5
	      STR=TAKESTR(12,IE)
	      read(STR,*,err=99,end=99)
     +      IT(K),JT(K),KT(K),LT(K),FT1(K),FT2(K),FT3(K),FT4(K),FT5(K)
	      NMUL(K)=0   
	      FT(K)=-(FT1(K)+FT2(K)+FT3(K)+FT4(K)+FT5(K)) 
	      RT(K)=180.d0 
	    end do
	    if(IPRINT.ge.8)then
	      PRINT *
            PRINT *,'*** DIHEDRALS '
	      write(*,*)'  Sum(Cosf)**n type'
	      do K=NRTT+1,NRTT+NTT
              PRINT"(4I6,4(1X,A4),' ->',5f8.2)",
     +        IT(K),JT(K),KT(K),LT(K),
     +        NM(IT(K)+NX),NM(JT(K)+NX),NM(KT(K)+NX),NM(LT(K)+NX),
     +        FT1(K),FT2(K),FT3(K),FT4(K),FT5(K)        
	      end do
	    end if
          NRTT         = NRTT+NTT
	end if
      else if(NMF.eq.'Impro'.or.NMF.eq.'impro'.or.NMF.eq.'IMPRO')then	
*   2.8.3 Improper dihedrals  (impro)
        STR=TAKESTR(12,IE)
	read(STR,*,err=99,end=99)NTT       
	NRT(NTYP)=NRT(NTYP)+NTT
	if(NTT.gt.0)then
	  do K=NRTT+1,NRTT+NTT  
	    ITORS(K)=-1
	    STR=TAKESTR(12,IE)
	    read(STR,*,err=99,end=99)
     +      IT(K),JT(K),KT(K),LT(K),RT(K),FT(K)
	    NMUL(K)=0   
	  end do
	  if(IPRINT.ge.8)then
	    PRINT *
            PRINT *,'*** Improper DIHEDRALS '
	    do K=NRTT+1,NRTT+NTT
              PRINT"(4I6,4(1X,A4),' ->',5f8.2)",
     +        IT(K),JT(K),KT(K),LT(K),
     +        NM(IT(K)+NX),NM(JT(K)+NX),NM(KT(K)+NX),NM(LT(K)+NX),
     +        RT(K),FT(K)        
	    end do
	  end if
          NRTT         = NRTT+NTT
        end if
      else if(NMF.eq.'Hydro'.or.NMF.eq.'hydro'.or.NMF.eq.'HYDRO')then
*  2.8.4 Hydrogen bonds    ( additional 10-12 potential)
C       In the input file one can specify names of donors and acceptors
C       of hydrogen bonds. These names should be present in file
C       PATHDB/hydro.bonds, together with parameters of 10-12 potential  
	STR=TAKESTR(12,IE)    
	I=0
	read(STR,*,err=99,end=99)NHT
	if(NHT.gt.0)NHDM=NHDM+NSPEC(NTYP) 
	if(NHT.gt.0.and.IPRINT.ge.6)write(*,*)'  hydrogen bonds:'
	do I=1,NHT
	  STR=TAKESTR(12,IE)           
	  read(STR,*,err=99,end=99)NAMH,ITPH
	  do J=1,NSTS
            IS=NX0+J 
	    if(NM(IS).eq.NAMH)then 
	      IHYD(IS)=ITPH
	      NHYDB=NHYDB+1 
	      if(IPRINT.ge.6)write(*,*)NAMH,' site ',IS,' typ H: ',ITPH
	    end if
	  end do
	end do
      else if(NMF.eq.'Speci'.or.NMF.eq.'speci'.or.NMF.eq.'SPECI')then
*  2.8.5 List of specific 1-4 interactions
C     These special LJ parameters are used only if specified
C     below atoms are "1-4" bound 
 	STR=TAKESTR(12,IE)
	read(STR,*,err=99,end=99)IADLJ 
	if(IPRINT.ge.6)write(*,*)' Overwrite ',IADLJ,' LJ parameters'
	do I=NNAD+1,NNAD+IADLJ
	  STR=TAKESTR(12,IE)
	  read(STR,*,err=99,end=99)ILJ(I),SIGAD(I),EPAD(I) 
	  ILJ(I)=ILJ(I)+NX
	  if(IPRINT.ge.8)write(*,'(i4,2x,a4,2x,2f12.3)')
     +    ILJ(I),NM(ILJ(I)),SIGAD(I),EPAD(I)
	end do   
	NNAD=NNAD+IADLJ
	if(NNAD.gt.NNADM)stop ' increase NNADM'
*   2.8.6  Other cases
      else if(NMF(1:3).eq.'End'.or.NMF(1:3).eq.'end'.or.
     +  NMF(1:3).eq.'END')then
	  go to 20	
      else
	  LD=LENG(STR,73)
	  write(*,*)' !!! Potential for ',STR(I:LD),' not implemented'
	  write(*,*)' !!! Molecule ',NAMOL(NTYP)
      end if
      go to 100
*
*  2.9   Addresses for bonds, angles and dihedral angles
*  -----------------------------------------------------
 20   IADB(NTYP)   = NRBB-NRB(NTYP)+1
      IADA(NTYP)   = NRAA-NRA(NTYP)+1
      IADT(NTYP)   = NRTT-NRT(NTYP)+1
      NX=NX+NSTS
      close (12)
      return                      
 99	IE=99
	STR=TAKESTR(12,IE)
      write(*,*)' This message should never appear on the screen ...'
 999  write(*,*)'!!! Molecule ',NAMN,' not found in the data base'
      write(*,*)'    File not found: ',FIL
      stop
      end
*================================================================
*
*    3. Read and check a line from input file
*    ----------------------------------------
*
C    This subroutine reads a line from Fortran input file 
C    defined by "KAN" (i.e. opened with parameter unit=KAN)
C    It returns the line if it is not started with # (commentaries); 
C    otherwise it reads and check the next line.
C    Parameter IE:
C    If input value of IE is NOT 99, the subroutine performs normal action
C    (see above) and return IE=0 (normal exit) or IE=-1 (read error)
C    If input value of IE=99, the subroutine is used for diagnostics:
C    it reports the file name and the line number where input error occur.
C    
C
C============== TAKESTR =========================================
C
*   3.1 Definitions
      character*128 function TAKESTR(KAN,IE)
      character*128 AUX, fn*32 
      integer LCOUNT(256)   
      data LCOUNT /256*0/
      save AUX,LCOUNT
*   3.2 Normal action           
      if(IE.eq.99)go to 10
C  LCOUNT variable count line number (including commentaries) 
 1	LCOUNT(KAN)=LCOUNT(KAN)+1
 	read(KAN,'(a128)',err=10,end=20)AUX
	if(AUX(1:1).eq.'#')go to 1
	TAKESTR=AUX 
	IE=0
	return
*   3.3 Error diagnostic
 10	write(*,*)'!!! error in input file '   
	if(KAN.eq.5)then
	  write(*,*)' Standard input ' 
	else
	  inquire(unit=KAN,name=fn) 
	  write(*,*)' File ',fn
	end if
	write(*,*)' in line ',LCOUNT(KAN)
	write(*,*)AUX
	stop
*    3.4 End of file case (is not signaled if input IE < 0 )
 20	if(IE.ge.0)then
 	  write(*,*)'!!! end of file reached' 
	  if(KAN.eq.5)then
	    write(*,*)' Standard input ' 
	  else
	    inquire(unit=KAN,name=fn) 
	    write(*,*)' File ',fn
	  end if
	  stop
	end if  
	TAKESTR='  ' 
	IE=-1
	return
	end 
*=====================================================================
*
*   4. Read hydrogen bonds parameters
*   ---------------------------------
C
C   The subroutine set up arrays for hydrogen bonds
C   It uses list of donors and acceptors obtained from the .mol file 
C   and pick up corresponding parameters from the hydro.bond file
C
C================== HYDRO ==============================================
C                                                                       
      subroutine HYDRO
      include "prcm.h"
      character FIL*128,STR*128,NAMN*12,TAKESTR*128
*   4.1 Open hydrogen bonds database
C  This is name of file containing information of hydrogen bond parameters 
      NAMN = 'hydro.bonds'
      LD=LENG(PATHDB,116)
      FIL(1:LD)=PATHDB(1:LD)
      FIL(LD+1:LD+1)='/'
      LN=LENG(NAMN,12)
      LF=LD+LN+1
      FIL(LD+2:LF)=NAMN(1:LN)
      IE = 0
      open(unit=13,file=FIL(1:LF),status='old',err=100)
      STR=TAKESTR(13,IE)
      I=0
      read(STR,*,err=99,end=99)NLINES
*   4.2 Read in hydrogen bonds parameters
C   Hydrogen bonds are described by potential
C   V(I,J) = CHYD(I,J)/Rij**12 - DHYD(I,J)/Rij**10
C   Here: just read arrays CHYD and DHYD 
      do I=1,NLINES
	STR=TAKESTR(13,IE)
	read(STR,*,err=99,end=99)IH,JH,CHYD(IH,JH),DHYD(IH,JH)
	CHYD(JH,IH)=CHYD(IH,JH)
	DHYD(JH,IH)=DHYD(IH,JH)
	if(IH.gt.NHYD.or.JH.gt.NHYD)stop ' increase NHYD'
      end do 
      close (13)
      if(IPRINT.ge.5)write(*,*)
     +' read file with hydrogen bond parameters ',NAMN
      return
*  4.3  Case of error
 99   IE=99
      STR=TAKESTR(13,IE)
      write(*,*)' Kernel panic ...'
 100  write(*,*)' file ',fil,' not found'
      return
      end
*
*   5. A few specific procedures
*   ----------------------------
*   5.1 Calculate covalent angles from configuration in .mol file
C    
C=============== ANGLG =============================================
C
      FUNCTION ANGLG(R,I,J,K,N)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION R(N,3)
      RIJ=0.
      RJK=0.
      SCL=0.
      DO M=1,3
	RIJ=RIJ+(R(I,M)-R(J,M))**2
        RJK=RJK+(R(J,M)-R(K,M))**2
        SCL =SCL +(R(I,M)-R(J,M))*(R(K,M)-R(J,M))
      end do
      ANGLG = DACOS(SCL/sqrt(RIJ*RJK))
      RETURN
      END
*
*   5.2 Do read(STR,*)(NAME(I),I=1,N) where STR and NAME are characters
*   -------------------------------------------------------------------
C   (only because some stupid compilers do not understand this)
C========= PARSES ============================
C
	subroutine PARSES(STR,CH,N,L,ie)
	character STR(L),CHR
	character*32 CH(N)
	logical LI
C  LCH=32 is length of NAMOL in prcm.h
	LCH=32
	do I=1,N
	   do J=1,LCH
	      CH(I)(J:J)=' '
	   end do
	end do
	LI=.false.
	M=0
	ie=0
	do I=1,L
	  CHR=STR(I)
	  ICHR=ichar(CHR)
	  if(LI)then
C  Delimiters - space, null and tab
	    if(CHR.eq.' '.or.ICHR.eq.0.or.ICHR.eq.9)then		
	      LI=.false.
	    else
	      IC=IC+1
	      if(IC.le.LCH)CH(M)(IC:IC)=CHR
	    end if
	  else
	     if(CHR.ne.' '.and.ICHR.ne.0.and.ICHR.ne.9)then
		M=M+1
		if(M.gt.N)return
		CH(M)(1:1)=CHR
		IC=1
		LI=.true.
	     end if
	  end if
	end do
	if(M.lt.N)ie=1
	return
	end

	
	   
