*
*=============== SHEJK ===========================================
*
*   SHAKE algorithm for rigid molecules
*   --------------------------------------
C   This algorithm provides integration of equations of motions
C   with constraints: All bond lengths in the given molecule (IMOL) are fixed
C   The algorithm is based on Lagrange formalism of solution of 
C   equations of motion with constrains
C
      SUBROUTINE SHEJK(IMOL)
*
*   4.1  Definitions 
*   ----------------
      include "mdee.h"
      LOGICAL MOVING(NTOT),MOVED (NTOT),DONE
      PARAMETER (MAXIT=2000)
*
*   4.2  Prepare
*   ------------
*   4.2.1 Calculate some references
      ITYP      = ITM(IMOL)              ! type of the molecule
      NRBS      = NRB(ITYP)              ! number ob bonds (constraints)
      if(ISHEJK(ITYP).le.0.or.NRBS.le.0)return
      TOL2      = TOLER**2               ! tolerance parameter
      NBBEG     = IADB(ITYP)             ! first bond in the molecule
      NBEND     = NBBEG+NRBS-1           ! last bond
      ISHF      = ISADDR(ITYP)+NSITS(ITYP)*(IMOL-IADDR(ITYP)-1)
      ISBEG     = ISHF+1                 ! first atom in the molecule
      ISEND     = ISHF+NSITS(ITYP)       ! last atom
*   4.2.2 Remember unconstrained positions
C   Arrays for forces FX,... are used as a temporary storage
C   Forces are not needed at this step
      do I=ISBEG,ISEND
        FX(I)=SX(I)
        FY(I)=SY(I)
        FZ(I)=SZ(I)
      end do
*   4.2.3 Set up logical arrays controlling constraints
      DO M      = NBBEG,NBEND
	LI	= IB(M)+ ISHF
	LJ	= JB(M)+ ISHF
        MOVING(LI) = .FALSE.
        MOVED (LI) = .TRUE.
        MOVING(LJ) = .FALSE.
        MOVED (LJ) = .TRUE.
      END DO! OF M
*
*  4.2.4  Begin iterations:
*  -----------------   
      ITER      = 0
      DONE      = .FALSE.
C  Point to returm:
    1 CONTINUE
C  Maximum iterations exceed
      IF(ITER.GT.MAXIT) GO TO 1000
*
*  4.3  Adjustment of constraints 
*  ------------------------------
*  4.3.1 Begin cycle over the bonds
      IF(.NOT.DONE) THEN
C   "DONE" will be .true. at the end of iteration if all constrains satisfied
        DONE      = .TRUE.
        DO M      = NBBEG,NBEND
          I       = IB(M)+ISHF
          J       = JB(M)+ISHF
C  MOVED=.true. means that the atom should be checked:
C  either if the bond was moved after the constrain was adjusted 
C  or the constrain was not adjusted at all      
          if(MOVED(I).OR.MOVED(J)) then
*  4.3.2 Calculate deviation
            AMAS      = 1.D0/MASSDI(I)
            BMAS      = 1.D0/MASSDI(J)
            BNDSQ     = RB(M)**2
            BXN       = SX(I)-SX(J)
            BYN       = SY(I)-SY(J)
            BZN       = SZ(I)-SZ(J)
	    call PBC(BXN,BYN,BZN)
            BSQ       = BXN**2+BYN**2+BZN**2
            DIFFSQ    = BNDSQ-BSQ
            if(IPRINT.ge.10)write(*,'(2i5,f9.6,4(2x,l1),i5)')
     +I,J,sqrt(BSQ),moved(I),moved(j),moving(I),moving(j),ITER
*   4.3.3 Deviation is still large - adjust the atoms
            IF(DABS(DIFFSQ).GT.(BNDSQ*TOL2))THEN
C   OX.. Constrained positions from step n
              DXN       = OX(I)-OX(J)
              DYN       = OY(I)-OY(J)
              DZN       = OZ(I)-OZ(J)
	      call PBC(DXN,DYN,DZN)
              VDOT      = BXN*DXN+BYN*DYN+BZN*DZN
              IF(VDOT.LT.BNDSQ*BDTOL) THEN
C  This happens if the deviation increased....
      write(*,*)' --> constrained failure for atoms ',I,J
      WRITE(*,66) 'old     :',OX(I),OY(I),OZ(I),OX(J),OY(J),OZ(J)
      WRITE(*,66) 'unconstr:',FX(I),FY(I),FZ(I),FX(J),FY(J),FZ(J)
      WRITE(*,66) 'current :',SX(I),SY(I),SZ(I),SX(J),SY(J),SZ(J)
      write(*,*)'OLD DISTANCE: ',DSQRT((OX(I)-OX(J))**2
     X                             +(OY(I)-OY(J))**2
     X                             +(OZ(I)-OZ(J))**2)
      write(*,*)'unconstrained:',DSQRT((FX(I)-FX(J))**2
     X                             +(FY(I)-FY(J))**2
     X                             +(FZ(I)-FZ(J))**2)
      write(*,*)'NEW DISTANCE: ',DSQRT((SX(I)-SX(J))**2
     X                             +(SY(I)-SY(J))**2
     X                             +(SZ(I)-SZ(J))**2)
*
   66 FORMAT(1X,a,3(1X,F10.6),3X,3(1X,F10.6))
CD    STOP 'CONSTRAINED FAILURE'
              END IF
              RMA       = MASSDI(I)
              RMB       = MASSDI(J)
              GAB       = DIFFSQ/(2.D0*(RMA+RMB)*VDOT)
              DXN       = DXN*GAB
              DYN       = DYN*GAB
              DZN       = DZN*GAB
C    Moving the atoms
              SX(I)     = SX(I)+RMA*DXN
              SY(I)     = SY(I)+RMA*DYN
              SZ(I)     = SZ(I)+RMA*DZN
              SX(J)     = SX(J)-RMB*DXN
              SY(J)     = SY(J)-RMB*DYN
              SZ(J)     = SZ(J)-RMB*DZN
C    Mark atom as MOVING during the iteration
              MOVING(I) = .TRUE.
              MOVING(J) = .TRUE.
C    Next iteration required because some atoms were moved
              DONE      = .FALSE.
            END IF
          END IF
        END DO! OF M - all bonds
*
*  4.4  Check that all constraints are satisfied
*  ---------------------------------------------
*  Obs!    These are separate loops ( Reccurence possible )
C  Set OK to all the bonds  
        DO M      = NBBEG,NBEND
          I       = IB(M)+ISHF
	  J	  = JB(M)+ISHF
	  MOVED(I)  = .false.
          MOVED(J)  = .false.
        END DO! OF M
C  If the atom was moved during the last iteration, mark it as requiring check
C  at the next iteration
        DO M      = NBBEG,NBEND
          I       = IB(M)+ISHF
	  J	  = JB(M)+ISHF
          if(MOVING(I))MOVED(I)=.true.
          if(MOVING(J))MOVED(J)=.true.
C  ... and set MOVING as false for the next iteratio
          MOVING(I) = .FALSE.
          MOVING(J) = .FALSE.
        END DO! OF M
        ITER      = ITER + 1
C  Continue iterations
        GO TO 1
      END IF  !(.not.DONE
*
*   4.5 Calculate corrections to the velocities and contribution to virial
*   ----------------------------------------------------------------------
 1000 CONTINUE
*
      IF(ITER.GE.MAXIT) THEN
        write(*,*)'??? NO CONVERGENCE AFTER MAXIT INTERATIONS'
        write(*,'(a,i6,a,i6,a,f12.6,a,f12.6/)')
     +'??? BOND BETWEEN ATOMS ',I,' AND',J,'  Reqv = ',
     +dsqrt(BNDSQ),' Rcalc = ',dsqrt(BSQ)
      END IF
C   velocity correction
      DO I      = ISBEG,ISEND
        DIX       = (SX(I)-FX(I))/(MASSDI(I)*TSTEP)   
        DIY       = (SY(I)-FY(I))/(MASSDI(I)*TSTEP)
        DIZ       = (SZ(I)-FZ(I))/(MASSDI(I)*TSTEP)
        VIRA(1)   = VIRA(1)+DIX*OX(I)/TSTEP
        VIRA(2)   = VIRA(2)+DIY*OY(I)/TSTEP
        VIRA(3)   = VIRA(3)+DIZ*OZ(I)/TSTEP
        VX(I)     = VX(I)+DIX
        VY(I)     = VY(I)+DIY
        VZ(I)     = VZ(I)+DIZ
      END DO! OF N
      RETURN
      END
*
*=============== BNDLST ===========================================
*
      SUBROUTINE BNDLST
*
      include "mdee.h" 
	parameter (RMX2=5.**2)
*                                 
	do I=1,NSTOT
	  NNBB(I)=0
	  do IS=1,NBDMAX
           INBB(I,IS)=0
	  end do
	end do       
*	IBEG=TASKID+1 
*	NRN=NUMTASK  
*	if(NRN.gt.NTYPES)NRN=NTYPES
	do ITYP=1,NTYPES
	  NRNOD(ITYP)=mod(ITYP-1,NUMTASK)
	end do  
*	if(IBEG.gt.NTYPES)go to 101
	do ITYP=1,NTYPES
*           
        NNMAX     = -99999
        NSP       = NSPEC(ITYP)     ! num of mol of this type
        NSS       = NSITS(ITYP)     ! num of sites on a mol of this type
        NRBS      = NRB(ITYP)       ! num of bonds 
        NRAS      = NRA (ITYP)      ! num of angles 
        NRTS      = NRT(ITYP)       ! num of torsions
*
        NBBEG     = IADB(ITYP)
        NBEND     = NBBEG+NRBS-1
        NABEG     = IADA(ITYP)
        NAEND     = NABEG+NRAS-1
        NTBEG     = IADT(ITYP)
        NTEND     = NTBEG+NRTS-1                       
*             
*
        ISBEG     = ISADR(ITYP)+1
        ISEND     = ISADR(ITYP+1)
        ISHF      = ISADR(ITYP)  ! shift of global site num. relatevely local
	ISHA	    = ISADDR(ITYP)-ISHF
*
*  Double sum over sites 
*  
	 DO IS=ISBEG,ISEND-1
	    DO JS=IS+1,ISEND   
*  IS, JS - site number
*  Bonds    
	      if(.not.L15NB(ITYP))go to 100
	      I=ISHA+IS
	      J=ISHA+JS                
	      DX=SX(I)-SX(J)
	      DY=SY(I)-SY(J)
	      DZ=SZ(I)-SZ(J)
	      call PBC(DX,DY,DZ)                
*   atoms are too far
              RRR=DX**2+DY**2+DZ**2
              if(RRR.gt.RMX2)go to 150
	      do IIS=NBBEG,NBEND
		II=IB(IIS)+ISHF
		JJ=JB(IIS)+ISHF
		if(II.eq.IS.and.JJ.eq.JS)go to 100
		if(II.eq.JS.and.JJ.eq.IS)go to 100
	      end do
*  Angles
	      do IIS=NABEG,NAEND
		II=IA(IIS)+ISHF
		KK=KA(IIS)+ISHF
		if(II.eq.IS.and.KK.eq.JS)go to 100
		if(II.eq.JS.and.KK.eq.IS)go to 100
	      end do          
*  Torsions      
	      do IIS=NTBEG,NTEND
		II=IT(IIS)+ISHF
		LL=LT(IIS)+ISHF 
		ITR=ITORS(ITYP)
		if((II.eq.IS.and.LL.eq.JS).or.(II.eq.JS.and.LL.eq.IS))
     +	then
		  if(ITR.eq.-1.or..not.L14NB(ITYP))then
		    go to 100
		  else
	  	    go to 90
	 	  end if
		end if
	      end do
*  Pass all checks - non-bound 
            go to 150
*  These atoms are 1-4 bound (Scaling LJ parameters may be used)
 90         do IIM=0,NSP-1
	        ISHIFT=ISADDR(ITYP)-ISADR(ITYP)+IIM*NSS 
		ISA=IS+ISHIFT 
		JSA=JS+ISHIFT 
	 	NNBB(ISA)=NNBB(ISA)+1      ! number of bound neigbours for this atom
		INBB(ISA,NNBB(ISA))=-JSA   ! bound neigbours ( - for 1-4 interactions)
		if(NNBB(ISA).gt.NBDMAX)then
		  write(*,*)
     +' list of bound atoms exceeded for atom ',ISA,' ',NM(ISA)
		  stop
		end if 
	 	NNBB(JSA)=NNBB(JSA)+1
		INBB(JSA,NNBB(JSA))=-ISA  
		if(NNBB(JSA).gt.NBDMAX)then
		  write(*,*)
     +' list of bound atoms exceeded for atom ',JSA,' ',NM(JSA)
		  stop
		end if 
	      end do 
	      go to 150
*   Atoms are 1-3 or 1-2 bounded. 
 100        do IIM=0,NSP-1
	        ISHIFT=ISADDR(ITYP)-ISADR(ITYP)+IIM*NSS
		ISA=IS+ISHIFT 
		JSA=JS+ISHIFT 
	 	NNBB(ISA)=NNBB(ISA)+1
		INBB(ISA,NNBB(ISA))=JSA
		if(NNBB(ISA).gt.NBDMAX)then
		  write(*,*)
     +' list of bound atoms exceeded for atom ',ISA,' ',NM(ISA)
		  stop
		end if 
	 	NNBB(JSA)=NNBB(JSA)+1
		INBB(JSA,NNBB(JSA))=ISA  
		if(NNBB(JSA).gt.NBDMAX)then
		  write(*,*)
     +' list of bound atoms exceeded for atom ',JSA,' ',NM(JSA)
		  stop
		end if 
	      end do
 150        continue
	    end do
	  end do
*----------------------------
 	  write(*,*)' list of bound atoms for ',NAME(ITYP),' prepared'
	end do   ! of ITYP      
 101	continue
CM call  GATHER(NRNOD)
CM
	if(IPRINT.ge.8)then
	  write(*,*)' list of bound atoms: '
	  do I=1,NSTOT
	    write(*,'(i5,i3,2x,40i5)')I,NNBB(I),(INBB(I,J),J=1,NNBB(I))
	  end do 
	end if
*----------------------------------------
      RETURN
      END   
